@@ -4,9 +4,13 @@
 
 # import json
 # import shutil
+import gzip
+import json
 import os
 import re
+import io
 import shutil
+import traceback
 # from pathlib import Path
 
 import objectUtils as obj
@@ -16,6 +20,7 @@ import file_search as search
 import string_utils as strUtils
 from secure_delete import secure_delete
 from threading import Thread
+import colemen_string_utils as strUtils
 
 # pylint: disable=line-too-long
 DEST_PATH_SYNONYMS = ["dest", "dst", "dest path", "destination", "destination path", "dst path", "target", "target path"]
@@ -50,9 +55,9 @@ def get_data(file_path, **kwargs):
 
     file_data = {}
     file_data['file_name'] = os.path.basename(file_path)
-    ext = os.path.splitext(file_data['file_name'])
-    file_data['extension'] = ext[1]
-    file_data['name_no_ext'] = os.path.basename(file_path).replace(file_data['extension'], '')
+    # ext = os.path.splitext(file_data['file_name'])
+    file_data['extension'] = get_ext(file_path)
+    file_data['name_no_ext'] = get_name_no_ext(file_path)
     file_data['file_path'] = file_path
 
     if 'dir_path' not in exclude:
@@ -69,6 +74,64 @@ def get_data(file_path, **kwargs):
     return file_data
 
 
+def get_ext(file_path):
+    file_name = os.path.basename(file_path)
+    ext = os.path.splitext(file_name)
+    return ext[1]
+
+
+def get_name_no_ext(file_path):
+    return os.path.basename(file_path).replace(get_ext(file_path), '')
+
+
+def decompress(file_obj, compression="gzip"):
+    file_list = _gen_path_list(file_obj)
+
+    if len(file_list) > 0:
+        for file in file_list:
+            if compression == "gzip":
+                _decompress_single_file_gzip(file)
+
+
+def compress(file_obj):
+    file_list = _gen_path_list(file_obj)
+    if len(file_list) > 0:
+        for file_path in file_list:
+            _compress_single_file_gzip(file_path)
+
+
+def _decompress_single_file_gzip(file_path):
+    temp_path = strUtils.format.file_path(f"{os.path.dirname(file_path)}/{get_name_no_ext(file_path)}.decomp")
+    contents = False
+    try:
+        with gzip.open(file_path, 'rb') as ip:
+            with io.TextIOWrapper(ip, encoding='utf-8') as decoder:
+                contents = decoder.read()
+                write.write(temp_path, contents)
+    except gzip.BadGzipFile:
+        print(f"File is not compressed: {file_path}")
+
+    if contents is not False:
+        # delete the compressed file
+        delete(file_path)
+        # rename the decompressed file
+        rename(temp_path, file_path)
+    return contents
+
+
+def _compress_single_file_gzip(file_path):
+    success = False
+    try:
+        contents = read.read(file_path)
+        with gzip.open(file_path, 'wb') as target_file:
+            target_file.write(contents.encode())
+            success = True
+    except OSError as error:
+        print(f"Failed to compress: {file_path} \n{error}")
+        print(traceback.format_exc())
+    return success
+
+
 def exists(file_path):
     '''
         Confirms that the file exists.
@@ -116,23 +179,73 @@ def delete(file_path, **kwargs):
         `method_name`: delete
     '''
     shred = obj.get_kwarg(["shred", "secure"], False, (bool), **kwargs)
-    success = False
-    if exists(file_path) is True:
-        try:
-            if shred is True:
-                secure_delete.secure_random_seed_init()
-                secure_delete.secure_delete(file_path)
-            else:
-                os.remove(file_path)
-        except PermissionError as error:
-            print(f"Failed to delete {file_path}, {error}")
-            success = True
-    else:
-        success = True
+    threading = obj.get_kwarg(["threading"], True, (bool), **kwargs)
+    max_threads = obj.get_kwarg(["max_threads"], 15, (int), **kwargs)
+    min_thread_threshold = obj.get_kwarg(["min thread threshold", "min files thread"], 100, (int), **kwargs)
 
-    if exists(file_path) is False:
-        success = False
-    return success
+    file_list = _gen_path_list(file_path)
+    print("file_list: ", json.dumps(file_list, indent=4))
+    # exit()
+    if len(file_list) > 0:
+        if threading is True and len(file_list) >= min_thread_threshold:
+            _thread_file_action(file_list, action="delete", max_threads=max_threads, min_thread_threshold=min_thread_threshold, shred=shred)
+            return
+
+        for file in file_path:
+            if isinstance(file, (dict)):
+                if 'file_path' in file:
+                    _delete_single_file(file['file_path'], shred)
+            if isinstance(file_path, (str)):
+                _delete_single_file(file_path, shred)
+    return True
+
+
+def _gen_path_list(file_obj):
+    '''
+        Generates a list of file paths from the mixed type object provided.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `file_obj` {str|list|dict}
+            The object to parse for file_paths.
+
+            It can be a single path, list of paths, list of file objects, or a single file object. 
+
+        Return {list}
+        ----------------------
+        A list of file paths, if none are found, the list is empty.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 12-10-2021 10:08:53
+        `memberOf`: file
+        `version`: 1.0
+        `method_name`: _gen_path_list
+    '''
+    file_list = []
+
+    if isinstance(file_obj, (str)) is True:
+        if exists(file_obj):
+            file_list.append(file_obj)
+
+    if isinstance(file_obj, (list)) is True:
+        for file in file_obj:
+            if isinstance(file, (str)) is True:
+                if exists(file):
+                    file_list.append(file)
+            if isinstance(file, (dict)) is True:
+                if "file_path" in file:
+                    if exists(file["file_path"]):
+                        file_list.append(file["file_path"])
+
+    if isinstance(file_obj, (dict)) is True:
+        if "file_path" in file_obj:
+            if exists(file_obj["file_path"]):
+                file_list.append(file_obj["file_path"])
+    return file_list
 
 
 def import_project_settings(file_name):
@@ -263,94 +376,123 @@ def copy(src, dest=False, **kwargs):
 
     if threading is True:
         if len(copy_list) >= min_thread_threshold:
-            _thread_copy_files(copy_list, max_threads=max_threads, min_thread_threshold=min_thread_threshold)
+            _thread_file_action(copy_list, action="copy", max_threads=max_threads, min_thread_threshold=min_thread_threshold)
             return
 
     _copy_files_from_array(copy_list)
     # print(f"copy_list: {copy_list}")
 
 
-def _copy_files_from_array(files):
-    for file in files:
+def _copy_files_from_array(file_list):
+    for file in file_list:
         os.makedirs(os.path.dirname(file['dst_path']), exist_ok=True)
         shutil.copy2(file['src_path'], file['dst_path'])
     return True
 
 
-def _thread_copy_files(files, **kwargs):
+def _delete_single_file(file_path, shred=False):
+    '''
+        Deletes or shreds a single file.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `file_path` {string}
+            Path to the file to delete.
+        [`shred`=False] {bool}
+            Securely shred the file (slower)
+
+        Return {bool}
+        ----------------------
+        True upon success, false otherwise.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 12-10-2021 10:18:06
+        `memberOf`: file
+        `version`: 1.0
+        `method_name`: _delete_single_file
+    '''
+    success = False
+    if exists(file_path) is True:
+        try:
+            if shred is True:
+                secure_delete.secure_random_seed_init()
+                secure_delete.secure_delete(file_path)
+            else:
+                os.remove(file_path)
+        except PermissionError as error:
+            print(f"Failed to delete {file_path}, {error}")
+            success = True
+    else:
+        success = True
+
+    if exists(file_path) is False:
+        success = False
+    return success
+
+
+def _delete_files_from_array(file_list, shred=False):
+    '''
+        Delete a list of files.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `file_list` {list}
+            A list of file paths to delete.
+
+        [`shred`=False] {bool}
+            Securely shred and delete the files (slower.)
+
+        Return {bool}
+        ----------------------
+        True if ALL files are deleted, False if any fail.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 12-10-2021 10:19:44
+        `memberOf`: file
+        `version`: 1.0
+        `method_name`: _delete_files_from_array
+    '''
+    success = True
+    for file in file_list:
+        if exists(file):
+            result = _delete_single_file(file, shred=shred)
+            if result is not True:
+                success = False
+    return success
+
+
+def _thread_file_action(file_list, **kwargs):
     max_threads = obj.get_kwarg(["max_threads"], 15, (int), **kwargs)
     min_thread_threshold = obj.get_kwarg(["min thread threshold", "min files thread"], 100, (int), **kwargs)
+    action = obj.get_kwarg(["action"], "copy", (str), **kwargs)
+    shred = obj.get_kwarg(["shred"], False, (bool), **kwargs)
 
-    if len(files) <= min_thread_threshold:
+    if len(file_list) <= min_thread_threshold:
         max_threads = 1
 
-    files_per_thread = round(len(files) / max_threads)
+    files_per_thread = round(len(file_list) / max_threads)
     threads = []
     for idx in range(max_threads):
         start_idx = files_per_thread * idx
         end_idx = start_idx + files_per_thread
-        if end_idx > len(files):
-            end_idx = len(files)
-        files_array = files[start_idx:end_idx]
-        threads.append(Thread(target=_copy_files_from_array, args=(files_array,)))
+        if end_idx > len(file_list):
+            end_idx = len(file_list)
+        files_array = file_list[start_idx:end_idx]
+        if action == "copy":
+            threads.append(Thread(target=_copy_files_from_array, args=(files_array,)))
+        if action == "delete":
+            threads.append(Thread(target=_delete_files_from_array, args=(files_array, shred)))
 
     for thread in threads:
         thread.start()
     for thread in threads:
         thread.join()
     return
-
-
-# files = [{
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\.picasa.ini",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\.picasa.ini"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211029151612812.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211029151612812.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211029151754458.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211029151754458.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030002727270.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030002727270.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030002838887.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030002838887.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030002845943.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030002845943.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030003001713.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030003001713.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030003008915.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030003008915.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030005014600.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030005014600.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030011919612.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030011919612.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030011927976.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030011927976.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030014348434.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030014348434.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030014355273.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030014355273.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030022210268.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030022210268.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030022219671.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030022219671.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030022304089.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030022304089.jpg"
-# }, {
-#     "src_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\sourceDirectory\\rendering\\20211030022401058.jpg",
-#     "dst_path": "C:\\Users\\Colemen\\Desktop\\TEST_FOLDER\\Mnemosyne Tests\\targetDirectory\\rendering\\20211030022401058.jpg"
-# }]
-# copy(files)
